// cmnd.cpp : Implementation of Ccmnd
#include "stdafx.h"
#include "Asp1503.h"
#include "cmnd.h"


/////////////////////////////////////////////////////////////////////////////
// Ccmnd
HRESULT Ccmnd::Activate()
{
   HRESULT hr = S_OK;
   CComVariant vt;
   CComPtr<IGetContextProperties> pProps; //Context Properties

   IDispatch* piDispatch = NULL;

   // get ObjectContext
   hr = CoGetObjectContext(IID_IObjectContext, (void **)&m_spObjectContext);
   if (FAILED(hr)) return hr;
   
   // get Context Properties
   hr = m_spObjectContext->QueryInterface( IID_IGetContextProperties, 
               (void**)&pProps );
   if (FAILED(hr)) return hr;

   // get Response property
   hr = pProps->GetProperty( _bstr_t("Response"), &vt ) ;
   if (FAILED(hr)) return hr;

   piDispatch = vt. pdispVal;
   hr = piDispatch->QueryInterface( IID_IResponse, 
                     (void**)&m_piResponse );
   if (FAILED(hr)) return hr;

   // get connection string
   CComPtr<IApplicationObject> pApplication;

   hr = pProps->GetProperty(_bstr_t("Application"),&vt);
   if (FAILED(hr)) return hr;

   piDispatch = vt. pdispVal;
   hr = piDispatch->QueryInterface( IID_IApplicationObject, 
                     (void**)&pApplication );
   if (FAILED(hr)) return hr;
  
   CComVariant vtConn;
   CComBSTR bstrName("connection");
   hr = pApplication->get_Value(bstrName,&vtConn);
   
   if (FAILED(hr)) return hr;

   m_bstrConnection = vtConn;

   return S_OK;
} 

BOOL Ccmnd::CanBePooled()
{
	return TRUE;
} 

void Ccmnd::Deactivate()
{
    m_piResponse.Release();
	m_spObjectContext.Release();
} 


STDMETHODIMP Ccmnd::showPages(VARIANT *pvarTypes)
{
   HRESULT hr;

   _CommandPtr pCommand = NULL;
   _ParameterPtr pprmPageType = NULL;
   _RecordsetPtr pPages = NULL;  

   // set command properties
   pCommand.CreateInstance(__uuidof(Command));
   pCommand->PutPrepared(true);
   pCommand->PutCommandText(L"select name from WebPage where page_type_cd = ?");
   pCommand->PutActiveConnection(_variant_t(m_bstrConnection));

   // create parameter
   pprmPageType = pCommand->CreateParameter(L"page_type_cd",adChar,adParamInput,3);
   pCommand->Parameters->Append(pprmPageType);

   // wrap with _variant_t for resource allocation and 
   // deallocation
   _variant_t vtArray (pvarTypes);

   VARTYPE vt = V_VT(&vtArray);

   // if ARRAY, process as SAFEARRAY
   if (vt & VT_ARRAY)
      {

      LONG lLBound, lUBound;
      SAFEARRAY * psa;
      _variant_t vtVal;

      // copy variant array to SAFEARRAY
      SafeArrayCopy(*(vtArray.pparray), &psa);

      // get dimensions of array
      // Get array bounds.
      hr = SafeArrayGetLBound(psa, 1, &lLBound);
      if (FAILED(hr))
          return hr;
      hr = SafeArrayGetUBound(psa, 1, &lUBound);
      if (FAILED(hr))
          return hr;

      // get each value, print out
      for (long l = lLBound; l <= lUBound; l++) {
	     hr = SafeArrayGetElement(psa, &l, &vtVal);
	     if (FAILED(hr))
	        return hr;

         try {
           // get parameter value and execute command
           pprmPageType->Value = vtVal;
           
           // process results
           pPages = pCommand->Execute(NULL,NULL,adCmdText);
           showPageInfo(pPages,vtVal);
           
         }
         catch (_com_error e) {
           m_piResponse->Write(_variant_t(e.Description()));
         }

      }

      // cleanup
      SafeArrayDestroy(psa);

   }

	return S_OK;
}

void Ccmnd::showPageInfo(_RecordsetPtr pPages, _variant_t vtVal) {

    // print out recordset
    m_piResponse->Write(_variant_t("<h3>"));
    m_piResponse->Write(vtVal);
    m_piResponse->Write(_variant_t("</h3>"));
    m_piResponse->Write(_variant_t("<TABLE border='1' cellpadding='5' cellspacing='0'>"));

    while (!(pPages->EndOfFile)) {
      m_piResponse->Write(_variant_t("<tr><td>"));
      m_piResponse->Write(pPages->Fields->GetItem(_variant_t("name"))->GetValue());
      m_piResponse->Write(_variant_t("</td></tr>"));
      pPages->MoveNext();
    }

    m_piResponse->Write(_variant_t("</table>"));
}

STDMETHODIMP Ccmnd::showPagesWithParm(BSTR bstrType, BSTR *bstrRows)
{

   _CommandPtr pCommand = NULL;
   _ParameterPtr pprmPageType = NULL;
   _ParameterPtr pReturn = NULL;
   _ParameterPtr pOutput = NULL;
   _RecordsetPtr pPages = NULL;  
   ParametersPtr pParms = NULL;

   _variant_t vtConn = m_bstrConnection;
   _bstr_t bstrReturn;
   bstrReturn = L"No rows were found for page type code";


   // set command properties
   pCommand.CreateInstance(__uuidof(Command));
   pCommand->PutPrepared(true);
   pCommand->PutCommandText(L"sp_getpages");
   pCommand->PutActiveConnection(vtConn);

   try {

     // create parameters
     pReturn = pCommand->CreateParameter(L"return",adInteger,adParamReturnValue,NULL);
     pCommand->Parameters->Append(pReturn);

     pOutput = pCommand->CreateParameter(L"output",adInteger,adParamOutput,NULL);
     pCommand->Parameters->Append(pOutput);

     pprmPageType = pCommand->CreateParameter(L"page_type_cd",adChar,adParamInput,3);
     pCommand->Parameters->Append(pprmPageType);

     //set value
     pprmPageType->Value = _variant_t(bstrType);
 
     pPages = pCommand->Execute(NULL,NULL,adCmdStoredProc);
     showPageInfo(pPages,_variant_t(bstrType));

     // release recordset to get output paramters
     pPages.Release();

     // get output and return parms
     _variant_t vtReturn;
     _variant_t vtRows;
     
     vtReturn = pReturn->GetValue();
     vtRows = pOutput->GetValue();

     if (vtReturn.lVal > 0)
        bstrReturn = L"No rows were found for page type code";
     else
        bstrReturn = L"Number of rows found was " + _bstr_t(vtRows);
   }
   catch (_com_error e) {
         m_piResponse->Write(_variant_t(e.Description()));
   }


   *bstrRows = bstrReturn;

   return S_OK;
}
